#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dict_hw.h"
#include "normxcorr_hw.h"

int dictGetCorrelations(ProcessingState ps, int icp, float *res) {
    int size = ps->fft_size;
    int size2 = size * size;

    cufftReal *cuda_result_buffer = (cufftReal*)ps->cuda_temp_buffer;
    float *cuda_final_buffer = cuda_result_buffer + CP_BLOCK * ps->fft_alloc_size;
    cudaMemcpy(res, cuda_final_buffer, size2*sizeof(cufftReal), cudaMemcpyDeviceToHost);
    
    return 0;
}

int dictGetCorrections(ProcessingState ps, float *res_x, float *res_y) {
    int ncp = ps->ncp;
    int points_alloc = ps->points_alloc_size;
    int ncp_alloc = ps->ncp_alloc_size;

    float *move_x = ps->points + 6 * points_alloc;
    float *move_y = move_x + points_alloc;

    cudaMemcpy2D(
	move_x, points_alloc * sizeof(float),
	ps->cuda_points, ncp_alloc * sizeof(float),
	ps->ncp * sizeof(float), 2,
	cudaMemcpyDeviceToHost
    );

    memcpy(res_x, move_x, ncp * sizeof(float));
    memcpy(res_y, move_y, ncp * sizeof(float));

    return 0;
}

int dictGetLocalSum(ProcessingState ps, int icp, float *lsum, float *denom) {
    int size = ps->fft_size;
    int size2 = size*size;
    int alloc_size = ps->fft_alloc_size;
    
    if (!ps->use_cache) icp = 0;

    if (lsum) {
        cudaMemcpy(lsum, ps->cuda_lsum_cache + icp * alloc_size, size2*sizeof(float), cudaMemcpyDeviceToHost);
    }

    if (denom) {
	cudaMemcpy(denom, ps->cuda_denom_cache + icp * alloc_size, size2*sizeof(float), cudaMemcpyDeviceToHost);
    }

    return 0;
}

